@@ -57,6 +57,11 @@ def fake_get_auth_keystone(expected_os_options=None, exc=None,
            actual_kwargs['cacert'] is None:
             from swiftclient import client as c
             raise c.ClientException("unverified-certificate")
+        if auth_url.startswith("https") and \
+           auth_url.endswith("client-certificate") and \
+           not (actual_kwargs['cert'] and actual_kwargs['cert_key']):
+            from swiftclient import client as c
+            raise c.ClientException("noclient-certificate")
 
         return storage_url, token
     return fake_get_auth_keystone
@@ -73,6 +78,10 @@ class StubResponse(object):
         self.body = body
         self.headers = headers or {}
 
+    def __repr__(self):
+        return '%s(%r, %r, %r)' % (self.__class__.__name__, self.status,
+                                   self.body, self.headers)
+
 
 def fake_http_connect(*code_iter, **kwargs):
     """
@@ -87,17 +96,18 @@ def fake_http_connect(*code_iter, **kwargs):
 
         def __init__(self, status, etag=None, body='', timestamp='1',
                      headers=None):
-            self.status = status
+            self.status_code = self.status = status
             self.reason = 'Fake'
+            self.scheme = 'http'
             self.host = '1.2.3.4'
             self.port = '1234'
             self.sent = 0
             self.received = 0
             self.etag = etag
-            self.body = body
+            self.content = self.body = body
             self.timestamp = timestamp
-            self._is_closed = True
             self.headers = headers or {}
+            self.request = None
 
         def getresponse(self):
             if kwargs.get('raise_exc'):
@@ -155,6 +165,9 @@ def fake_http_connect(*code_iter, **kwargs):
         def getheader(self, name, default=None):
             return dict(self.getheaders()).get(name.lower(), default)
 
+        def close(self):
+            pass
+
     timestamps_iter = iter(kwargs.get('timestamps') or ['1'] * len(code_iter))
     etag_iter = iter(kwargs.get('etags') or [None] * len(code_iter))
     x = kwargs.get('missing_container', [False] * len(code_iter))
@@ -213,6 +226,7 @@ class MockHttpTest(unittest.TestCase):
             on_request = kwargs.get('on_request')
 
             def wrapper(url, proxy=None, cacert=None, insecure=False,
+                        cert=None, cert_key=None,
                         ssl_compression=True, timeout=None):
                 if storage_url:
                     self.assertEqual(storage_url, url)
@@ -220,18 +234,22 @@ class MockHttpTest(unittest.TestCase):
                 parsed, _conn = _orig_http_connection(url, proxy=proxy)
 
                 class RequestsWrapper(object):
-                    pass
+                    def close(self):
+                        pass
                 conn = RequestsWrapper()
 
-                def request(method, url, *args, **kwargs):
+                def request(method, path, *args, **kwargs):
                     try:
                         conn.resp = self.fake_connect()
                     except StopIteration:
                         self.fail('Unexpected %s request for %s' % (
-                            method, url))
-                    self.request_log.append((parsed, method, url, args,
+                            method, path))
+                    self.request_log.append((parsed, method, path, args,
                                              kwargs, conn.resp))
                     conn.host = conn.resp.host
+                    conn.resp.request = RequestsWrapper()
+                    conn.resp.request.url = '%s://%s%s' % (
+                        conn.resp.scheme, conn.resp.host, path)
                     conn.resp.has_been_read = False
                     _orig_read = conn.resp.read
 
@@ -240,15 +258,15 @@ class MockHttpTest(unittest.TestCase):
                         return _orig_read(*args, **kwargs)
                     conn.resp.read = read
                     if on_request:
-                        status = on_request(method, url, *args, **kwargs)
+                        status = on_request(method, path, *args, **kwargs)
                         conn.resp.status = status
                     if auth_token:
                         headers = args[1]
                         self.assertEqual(auth_token,
                                          headers.get('X-Auth-Token'))
                     if query_string:
-                        self.assertTrue(url.endswith('?' + query_string))
-                    if url.endswith('invalid_cert') and not insecure:
+                        self.assertTrue(path.endswith('?' + query_string))
+                    if path.endswith('invalid_cert') and not insecure:
                         from swiftclient import client as c
                         raise c.ClientException("invalid_certificate")
                     if exc:
@@ -483,6 +501,9 @@ class CaptureOutput(object):
     def __eq__(self, other):
         return self.out == other
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __getattr__(self, name):
         return getattr(self.out, name)
 
@@ -528,9 +549,22 @@ class FakeKeystone(object):
         pass
 
 
-def _make_fake_import_keystone_client(fake_import):
-    def _fake_import_keystone_client(auth_version):
-        fake_import.auth_version = auth_version
-        return fake_import, fake_import
+class FakeStream(object):
+    def __init__(self, size):
+        self.bytes_read = 0
+        self.size = size
 
-    return _fake_import_keystone_client
+    def read(self, size=-1):
+        if self.bytes_read == self.size:
+            return b''
+
+        if size == -1 or size + self.bytes_read > self.size:
+            remaining = self.size - self.bytes_read
+            self.bytes_read = self.size
+            return b'A' * remaining
+
+        self.bytes_read += size
+        return b'A' * size
+
+    def __len__(self):
+        return self.size
